package soteria

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Soteria Documentation

Soteria is a comprehensive library for writing symbolic execution engines. The library is parametric and allows for defining one's own notion of symbolic value or solver, enabling arbitrary language-specific optimisations. Our goal is for symbolic execution engines to be easy to write, easy to read, highly customisable and efficient.

Manual contents

Library

Core components of Soteria, enabling symbolic execution, solver-interaction, user-friendly reporting, and more.

The two built-in symbolic value implementations, to get started; you can also define your own!

  • Soteria.Tiny_values Minimal natural integer based values. Suitable for toy examples.
  • Soteria.Bv_values Extensible bit-vector based values, with floating point support. Suitable for low-level languages.