package soteria

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Bv_values.Eval

module Make (Ext : Svalue.Value_ext) (V : sig ... end) : sig ... end

Normalisation of svalues over a built Svalue module V (the result of Svalue.Make). eval walks a value bottom-up and re-applies the simplifying smart constructors, which both simplifies and rebuilds any value that was assembled directly from raw nodes (e.g. read back from a cache).