package soteria

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module S_map.Lazy

Makes a symbolic map similar to Direct_access, but even in OX, if it is possible that the key is outside the map, only this path will be taken.

Parameters

module Symex : Symex.Base
module Key : Key(Symex).S

Signature

type 'a t
val empty : 'a t
val is_empty : 'a t -> bool
val syntactic_bindings : 'a t -> (Key.t * 'a) Seq.t
val syntactic_mem : Key.t -> 'a t -> bool
val syntactic_add : Key.t -> 'a -> 'a t -> 'a t
val syntactic_add_opt : Key.t -> 'a option -> 'a t -> 'a t
val pp' : ?key:(Format.formatter -> Key.t -> unit) -> ?ignore:((Key.t * 'a) -> bool) -> (Format.formatter -> 'a -> unit) -> Format.formatter -> 'a t -> unit
val pp : (Format.formatter -> 'a -> unit) -> Format.formatter -> 'a t -> unit
val show : ('a -> string) -> 'a t -> string
val find_opt : Key.t -> 'a t -> (Key.t * 'a option) Symex.t
val fold : ('acc -> (Key.t * 'a) -> 'acc Symex.t) -> 'acc -> 'a t option -> 'acc Symex.t