package soteria

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Soteria.Soteria_std

Standard library extensions and utilities for Soteria.

module Bimap : sig ... end

A bidirectional map implementation.

module Cmdliner_helpers : sig ... end

Cmdliner argument converters for file and directory paths.

module Compo_res : sig ... end

This module defines a three-way result type for compositional symbolic execution. Unlike standard Result.t which has two cases (Ok/Error), this type adds a third case Missing for bi-abduction scenarios where anti-frame inference is needed, or more generally to represent incompletenesses in the engine.

module Dynarray : sig ... end

Extensions to Stdlib.Dynarray with pretty-printing and JSON support.

module Effects : sig ... end

Effect used for bookkeeping, which keeps tracks of some information that can then be dumped "somewhere" (typically a file, configured externally), or ignored.

module Exn : sig ... end

Common exceptions that are not defined in the OCaml Stdlib

module Filename : sig ... end

Extensions to Stdlib.Filename with utility functions.

module Graph : sig ... end

An in-place mutable graph implementation with reachability and topological ordering.

module Hashset : sig ... end

A hash set implementation.

module Hashtbl : sig ... end

Extensions to Stdlib.Hashtbl with JSON support.

module Iarray : sig ... end
module Int : sig ... end

Extensions to Stdlib.Int with safe string conversion.

module Iter : sig ... end
module List : sig ... end

Extensions to Stdlib.List with utility functions.

module Map : sig ... end

Extensions to Stdlib.Map with pretty-printing support.

module Monad : sig ... end

Monad interfaces and implementations.

module Option : sig ... end

Extensions to Stdlib.Option with utility functions.

module Ordered_type : sig ... end

Extensions to Stdlib.Map.OrderedType with pretty-printing support.

module PatriciaTree : sig ... end

Persistent maps and sets keyed by an integer-encodable type, implemented as big-endian patricia tries.

module Resource_pool : sig ... end

Thread-safe resource pool for managing reusable resources.

module Result : sig ... end

Extensions to Stdlib.Result with conversion and extraction utilities.

module Reversible : sig ... end

Reversible computation abstractions.

module Seq : sig ... end

Extensions to Stdlib.Seq with utility functions.

module Sigs : sig ... end
module String : sig ... end
module Subprocess : sig ... end

Tracked OS subprocesses.

module Syntaxes : sig ... end

Monad syntax extensions for various container types.

module Sys : sig ... end

Extensions to Stdlib.Sys with utility functions.

module Unix : sig ... end
module Write_once : sig ... end
module Z : sig ... end