package soteria

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Changelog

All notable changes to Soteria, Soteria Rust and Soteria Rust are recorded here. The three are released together under a single version number. The format is based on Keep a Changelog; see RELEASING.md for the versioning policy and the release process.

Add entries for your change under ## [Unreleased] as part of your pull request.

[Unreleased]

[0.1.0] - 2026-06-27

This is our first release! We will be releasing regularly, under the 0.x series.

Added

  • Switched to OCaml 5.5.0!
  • soteria-c version and soteria-rust version now print the release version.

Changed

  • Minor performance improvements in Soteria Rust.